# Import modules
import os, sys

if 'ame_apy' not in sys.modules:
    try:
        from ame_apy import *
    except ImportError:
        print 'Unable to import AMESim API module.\nCheck the AME environment variable.'


import pylab

# Initialize API
AMEInitAPI(False)


# Create the system
print ''
print 'Creating "FlatTwin" system...'
AMECreateCircuit('FlatTwin')


# Add components
print ''
print 'Adding component to "FlatTwin" system...'

# Add "mass_friction1port" component
AMEAddComponent('mass_friction1port', 'mass_friction1port', (40, 99))

# Add "springdamper01" component
AMEAddComponent('springdamper01', 'springdamper01')

# Rotate and flip the component
AMERotateComponent('springdamper01')
AMEMoveComponent('springdamper01', (89, 99))

# Add "simple_crank" component
AMEAddComponent('simple_crank', 'simple_crank')

# Rotate and flip the component
AMERotateComponent('simple_crank')
AMEFlipComponent('simple_crank')
AMEMoveComponent('simple_crank', (138, 103))

# Add "rconnector" component
AMEAddComponent('rconnector', 'rconnector')

# Rotate and flip the component
AMERotateComponent('rconnector', 2)
AMEMoveComponent('rconnector', (176, 149))

# Add "simple_crank_2" component
AMEAddComponent('simple_crank', 'simple_crank_2')

# Rotate and flip the component
AMERotateComponent('simple_crank_2')
AMEMoveComponent('simple_crank_2', (229, 103))

# Add "springdamper01_2" component
AMEAddComponent('springdamper01', 'springdamper01_2')

# Rotate and flip the component
AMERotateComponent('springdamper01_2')
AMEMoveComponent('springdamper01_2', (287, 99))

# Add "rload01" component
AMEAddComponent('rload01', 'rload01')

# Rotate and flip the component
AMERotateComponent('rload01')
AMEMoveComponent('rload01', (187, 174))

# Add "mass_friction1port_2" component
AMEAddComponent('mass_friction1port', 'mass_friction1port_2')

# Rotate and flip the component
AMERotateComponent('mass_friction1port_2', 2)
AMEMoveComponent('mass_friction1port_2', (336, 99))


# Set submodels
print ''
print 'Setting submodels...'

# Set "MAS003" submodel to "mass_friction1port" component
AMEChangeSubmodel('mass_friction1port', 'MAS003', '$AME/libmec/submodels')

# Set "SD0000A" submodel to "springdamper01" component
AMEChangeSubmodel('springdamper01', 'SD0000A', '$AME/libmec/submodels')

# Set "CRANK0" submodel to "simple_crank" component
AMEChangeSubmodel('simple_crank', 'CRANK0', '$AME/libmec/submodels')

# Set "RCON00" submodel to "rconnector" component
AMEChangeSubmodel('rconnector', 'RCON00', '$AME/libmec/submodels')

# Set "CRANK0" submodel to "simple_crank_2" component
AMEChangeSubmodel('simple_crank_2', 'CRANK0', '$AME/libmec/submodels')

# Set "SD0000A" submodel to "springdamper01_2" component
AMEChangeSubmodel('springdamper01_2', 'SD0000A', '$AME/libmec/submodels')

# Set "RL00" submodel to "rload01" component
AMEChangeSubmodel('rload01', 'RL00', '$AME/libmec/submodels')

# Set "MAS003" submodel to "mass_friction1port_2" component
AMEChangeSubmodel('mass_friction1port_2', 'MAS003', '$AME/libmec/submodels')


# Do connections
print ''
print 'Connecting components...'

# Connect "mass_friction1port" and "springdamper01"
AMEConnectTwoPorts('mass_friction1port', 0, 'springdamper01', 0)

# Connect "springdamper01" and "simple_crank"
AMEConnectTwoPorts('springdamper01', 1, 'simple_crank', 1)

# Connect "simple_crank" and "rconnector"
AMEConnectTwoPorts('simple_crank', 0, 'rconnector', 1)

# Connect "rconnector" and "simple_crank_2"
AMEConnectTwoPorts('rconnector', 0, 'simple_crank_2', 0)

# Connect "rconnector" and "rload01"
AMEConnectTwoPorts('rconnector', 2, 'rload01', 0)

# Connect "simple_crank_2" and "springdamper01_2"
AMEConnectTwoPorts('simple_crank_2', 1, 'springdamper01_2', 0)

# Connect "springdamper01_2" and "mass_friction1port_2"
AMEConnectTwoPorts('springdamper01_2', 1, 'mass_friction1port_2', 0)


# Generate code
print ''
print 'Compiling "FlatTwin" system...'
AMEGenerateCode()


# Set parameter values
print ''
print 'Setting parameters to "FlatTwin" system...'

# Set 'mass_friction1port' parameters
AMESetParameterValue('v1@mass_friction1port', '0')
AMESetParameterValue('x1@mass_friction1port', '0')
AMESetParameterValue('mass@mass_friction1port', 'MASS')
AMESetParameterValue('coefv@mass_friction1port', '0')
AMESetParameterValue('wind@mass_friction1port', '0')
AMESetParameterValue('coul@mass_friction1port', '0')
AMESetParameterValue('stict@mass_friction1port', '0')
AMESetParameterValue('angle@mass_friction1port', '0')

# Set 'springdamper01' parameters
AMESetParameterValue('force0@springdamper01', '0')
AMESetParameterValue('srate@springdamper01', '1e5')
AMESetParameterValue('cdamp@springdamper01', '1e3')
AMESetParameterValue('G@springdamper01', '8.57e10')
AMESetParameterValue('sdiam@springdamper01', '20')
AMESetParameterValue('wdiam@springdamper01', '2')
AMESetParameterValue('stiffmode@springdamper01', '1')
AMESetParameterValue('ncoils@springdamper01', '10')

# Set 'simple_crank' parameters
AMESetParameterValue('theta@simple_crank', 'ANG')
AMESetParameterValue('rad@simple_crank', 'R')
AMESetParameterValue('length@simple_crank', 'L')
AMESetParameterValue('offset@simple_crank', '0')

# Set 'rconnector' parameters
# No parameter

# Set 'simple_crank_2' parameters
AMESetParameterValue('theta@simple_crank_2', 'ANG+OFF')
AMESetParameterValue('rad@simple_crank_2', 'R')
AMESetParameterValue('length@simple_crank_2', 'L')
AMESetParameterValue('offset@simple_crank_2', '0')

# Set 'springdamper01_2' parameters
AMESetParameterValue('force0@springdamper01_2', '0')
AMESetParameterValue('srate@springdamper01_2', '1e5')
AMESetParameterValue('cdamp@springdamper01_2', '1e3')
AMESetParameterValue('G@springdamper01_2', '8.57e10')
AMESetParameterValue('sdiam@springdamper01_2', '20')
AMESetParameterValue('wdiam@springdamper01_2', '2')
AMESetParameterValue('stiffmode@springdamper01_2', '1')
AMESetParameterValue('ncoils@springdamper01_2', '10')

# Set 'rload01' parameters
AMESetParameterValue('omega@rload01', '4000')
AMESetParameterValue('J@rload01', '10')
AMESetParameterValue('vis@rload01', '1')
AMESetParameterValue('coul@rload01', '0')
AMESetParameterValue('stict@rload01', '0')

# Set 'mass_friction1port_2' parameters
AMESetParameterValue('v1@mass_friction1port_2', '0')
AMESetParameterValue('x1@mass_friction1port_2', '0')
AMESetParameterValue('mass@mass_friction1port_2', 'MASS')
AMESetParameterValue('coefv@mass_friction1port_2', '0')
AMESetParameterValue('wind@mass_friction1port_2', '0')
AMESetParameterValue('coul@mass_friction1port_2', '0')
AMESetParameterValue('stict@mass_friction1port_2', '0')
AMESetParameterValue('angle@mass_friction1port_2', '0')


# Add global parameters
print ''
print 'Adding global parameters...'

AMEAddGlobalParameter('MASS', 'mass of the piston', 'ame_real_parameter', '0.5', '0.5', '0.1', '2', 'kg')
AMEAddGlobalParameter('BORE', 'bore of the piston', 'ame_real_parameter', '60', '60', '20', '100', 'mm')
AMEAddGlobalParameter('R', 'radius of crank', 'ame_real_parameter', '30', '30', '10', '50', 'mm')
AMEAddGlobalParameter('ANG', 'initial angular position of piston 1', 'ame_real_parameter', '0', '0', '0', '360', 'degree')
AMEAddGlobalParameter('OFF', 'difference in piston angles', 'ame_real_parameter', '180', '180', '0', '360', 'degree')
AMEAddGlobalParameter('L', 'length of connecting rod', 'ame_real_parameter', '40', '40', '25', '110', 'mm')


# Set run parameters
print ''
print 'Setting run parameters...'

AMESetRunParameter('stop_time_s', '0.1')
AMESetRunParameter('interval_s', '1e-3')


# Start simulation
print ''
print 'Running "FlatTwin" system simulation...'

AMERunSimulation()


# Save system
print ''
print 'Saving "FlatTwin" system...'
AMESaveCircuit()

# Import scipy and pylab modules
import scipy
import pylab	

# Get & plot values
print ''
print 'Plotting "x1@mass_friction1port" and "x1@mass_friction1port_2"...'

# get variable values
res_1 = AMEGetVariableValues('x1@mass_friction1port')
res_2 = AMEGetVariableValues('x1@mass_friction1port_2')

# Format values
x_1 = scipy.array(res_1)
x_2  =scipy.array(res_2)

# get variable title (these two variables have the same title)
variable_title = AMEGetVariableInfos('x1@mass_friction1port')[2]

# Plot variables
pylab.plot(x_1[:,0], x_1[:,1], label='x1@mass_friction1')
pylab.plot(x_2[:,0], x_2[:,1], label='x1@mass_friction1_2')
pylab.legend(loc='upper left')
pylab.xlabel('Time')
pylab.ylabel(variable_title)
pylab.grid(True)
print ''
print 'Close the plot to continue...'
pylab.show()


# Save and close the circuit
print ''
print 'Closing "FlatTwin" system...'
AMECloseCircuit(True)


# Close the API
AMECloseAPI()

